@@ -111,6 +111,16 @@ class TestRegistry:
         )
         assert len(registry) == total
 
+    def test_bool_empty(self):
+        assert not Registry()
+
+    def test_bool_not_empty(self):
+        registry = Registry().with_contents(
+            [(str(i), {"foo": "bar"}) for i in range(3)],
+            default_specification=Specification.OPAQUE,
+        )
+        assert registry
+
     def test_iter(self):
         registry = Registry().with_contents(
             [(str(i), {"foo": "bar"}) for i in range(8)],
@@ -819,7 +829,7 @@ class TestResolver:
         resource = Resource.opaque(contents={"foo": "baz"})
         once = [resource]
 
-        def retrieve(uri: str):
+        def retrieve(uri):
             return once.pop()
 
         resolver = Registry(retrieve=retrieve).resolver()
@@ -833,7 +843,7 @@ class TestResolver:
         resource = Resource.opaque(contents={"foo": "baz"})
         once = [resource]
 
-        def retrieve(uri: str):
+        def retrieve(uri):
             return once.pop()
 
         resolver = Registry(retrieve=retrieve).resolver()
@@ -956,6 +966,40 @@ class TestSpecification:
         )
         assert resource.id() == "urn:fixedID"
 
+    def test_detect_from_json_schema(self):
+        schema = {"$schema": "https://json-schema.org/draft/2020-12/schema"}
+        specification = Specification.detect(schema)
+        assert specification == DRAFT202012
+
+    def test_detect_with_no_discernible_information(self):
+        with pytest.raises(exceptions.CannotDetermineSpecification):
+            Specification.detect({"foo": "bar"})
+
+    def test_detect_with_non_URI_schema(self):
+        with pytest.raises(exceptions.CannotDetermineSpecification):
+            Specification.detect({"$schema": 37})
+
+    def test_detect_with_no_discernible_information_and_default(self):
+        specification = Specification.OPAQUE.detect({"foo": "bar"})
+        assert specification is Specification.OPAQUE
+
+    def test_detect_unneeded_default(self):
+        schema = {"$schema": "https://json-schema.org/draft/2020-12/schema"}
+        specification = Specification.OPAQUE.detect(schema)
+        assert specification == DRAFT202012
+
+    def test_non_mapping_detect(self):
+        with pytest.raises(exceptions.CannotDetermineSpecification):
+            Specification.detect(True)
+
+    def test_non_mapping_detect_with_default(self):
+        specification = ID_AND_CHILDREN.detect(True)
+        assert specification is ID_AND_CHILDREN
+
+    def test_detect_with_fallback(self):
+        specification = Specification.OPAQUE.detect({"foo": "bar"})
+        assert specification is Specification.OPAQUE
+
     def test_repr(self):
         assert (
             repr(ID_AND_CHILDREN) == "<Specification name='id-and-children'>"
